/* config.h - chan_ss7 configuration
 *
 * Copyright (C) 2006, Sifira A/S.
 *
 * Author: Anders Baekgaard <ab@sifira.dk>
 *
 * This file is part of chan_ss7.
 *
 * chan_ss7 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * chan_ss7 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with chan_ss7; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


/* Hunting policy. */
typedef enum { HUNT_ODD_LRU, HUNT_EVEN_MRU, HUNT_SEQ_LTH, HUNT_SEQ_HTL } hunting_policy;

/* Upper bounds only determined by installed hardware, use decent values */
#define MAX_E1_CONNECTOR_NO 8
#define MAX_CIC 256
#define MAX_LINKSETS 8
#define MAX_LINKS 16
#define MAX_LINKS_PER_LINKSET 16
#define MAX_LINKS_PER_HOST 16
#define MAX_SPANS_PER_HOST 16
#define MAX_SCHANNELS 4
#define MAX_IFS_PER_HOST 2
#define MAX_HOSTS 4

/* Echo cancellation constants */
enum {EC_DISABLED, EC_ALLWAYS, EC_31SPEECH};

typedef enum {STATE_UNKNOWN, STATE_ALIVE, STATE_DEAD} alivestate;
struct linkset {
  char* name;
  int n_links;
  struct link* links[MAX_LINKS_PER_LINKSET];
  char* context;
  char* language;
  hunting_policy hunt_policy;
  int dpc;
  int enabled;
  int use_connect;
  int enable_st;
  int subservice;
  int t35_value;
  int t35_action;
  int lsi;
  int n_schannels;
  int dni_chunk_limit;
  struct link* schannels[MAX_LINKS_PER_LINKSET];
  int first_cic, last_cic;
  int init_grs_done;		/* GRS sent? */
  /* Global circuit list. Protected by glock. */
  struct ss7_chan *cic_list[MAX_CIC];
  struct ss7_chan *idle_list;
};

struct link {
  char* name;
  int schannel;
  int first_zapid;
  unsigned long channelmask;
  int first_cic;
  int sls;
  int enabled;
  int send_sltm;
  int linkix;
  int echocancel;
  int echocan_taps;
  int echocan_train;
  struct linkset* linkset;
  struct host* on_host;
  struct receiver* receiver;
};

struct ipinterface {
  char* name;
  struct in_addr addr;
};

struct receiver {
  int n_targets;
  int receiverix;
  struct {
    struct host* host;
    struct ipinterface* inf;
  } targets[2*MAX_HOSTS];
};

struct host {
  char* name;
  /* Hosts point code */
  int opc;
  /* Destination point codes */
  int dpc[MAX_LINKSETS];
  /* Default linkset */
  struct linkset* default_linkset;
  /* IP interfaces */
  int n_ifs;
  struct ipinterface ifs[MAX_IFS_PER_HOST];
  int n_spans;

  /* E1/T1 connections */
  struct {
    struct link* link;
    int connector;
  } spans[MAX_SPANS_PER_HOST];

  /* Receivers, per link */
  int n_receivers;
  struct receiver receivers[MAX_LINKS_PER_HOST];

  alivestate state;
  int has_signalling_receivers;
  int enabled;
};

extern int n_linksets;
extern struct linkset linksets[MAX_LINKSETS];

extern struct host* this_host;
extern struct linkset this_hosts_linkset;

extern struct link links[];
extern int own_pc;

extern int clusterlistenport;

int load_config(int reload);
void destroy_config(void);
struct linkset* find_linkset_for_dpc(int pc);
struct linkset* lookup_linkset(char* name);
struct host* lookup_host_by_addr(struct in_addr);
struct host* lookup_host_by_id(int hostix);

